#!/bin/bash
#
# Linux Compatablity Tool Endian errors in global variable declarations checker
#  
#  Module: chkendian_globals.bash
#
#  Purpose: parse the objdump.txt file for info about global variables
#
#  Usage:
#        chkendian_globals.bash
#
#  Inputs:
#    $DB environment variable
#    $EXEFILE environment variable
#
#  Outputs:
#    tGlobalVarDef.tbl
#
#  Notes:
#    EXENAME is the image basename
#    EXEFILE is the full image name and path
#   

if [[ -z $DB ]]
then
    echo "Missing assignment"
    echo "    DB environment variable not set"
    exit 1
fi

if [[ -z $EXEFILE ]]
then
    echo "Missing assignment"
    echo "    EXEFILE environment variable not set"
    exit 1
fi

if [[ -z $LCTBIN ]]
then
    echo "Missing assignment"
    echo "    LCTBIN environment variable not set"
    exit 1
fi

if [[ -z $OBJDUMP ]]
then
    echo "Missing assignment"
    echo "    OBJDUMP environment variable not set"
    exit 1
fi

grep " \.bss" $OBJDUMP  |grep -v "object\.2"  | 
			      grep -v " stdout$"    |
			      grep -v " stderr$"    |
                              grep -v " stdout@@GLIBC_2.0$" |
                              grep -v " stderr@@GLIBC_2.0$" |
                              grep -v "@@GLIBC_2.0$" |
			awk '{  flg=0;
			varname = $NF
                        if ( $2 != "g" )        flg=1;
                        if ( $2 == ".bss" )     flg=1;
                        if ( $3 == "DO" )       flg=1;
                        if ( $5 == "00000000" ) flg=1;
                        if ( $6 == "" )         flg=1;
                        sub("[.][0-9]+$","", varname)
                        if ( flg == 0 ) 
                        print $1" " $5" " varname }' > $DB/objdump.tmp
                           

grep " \.data" $OBJDUMP  | awk '{ flg=0;
			varname = $NF
                        if ( $2 == ".data" )       flg=1;
                        if (substr($0,0,1) == " ") flg=1;
                        if ( $5 == "00000000" )    flg=1;
                        if ( $4 == "00000000" )    flg=1;
                        if ( $1 == "Contents" )    flg=1;
                        if ( $1 == "Disassembly" ) flg=1;
                        sub("[.][0-9]+$","", varname)
                        if ( flg == 0 ) print $1" " $5" "  varname " \""$0"\""}' >> $DB/objdump.tmp

[ $LCTSTAT ] && echo "	LCT 0% - `date`" >> $LCTSTAT

egrep -v " vtable | typeinfo " $DB/objdump.tmp > $DB/objdump.1

${LCTBIN}/get_global_var_def_info.exe  $DB/ $EXEFILE

[ $LCTSTAT ] && echo "	LCT 50% - `date`" >> $LCTSTAT

#${LCTBIN}/get_global_var_ref_info.bash
${LCTBIN}/getGlobalVarRefInfo.exe 

[ $LCTSTAT ] && echo "	LCT 100% - `date`" >> $LCTSTAT

#rm -f $DB/objdump.1 $DB/objdump.2 $DB/objdump.3

