#!/bin/bash
#
# Linux Compatablity Tool intrinsics usage checker
#


if [[ -z ${LCTBIN} ]]
then
    echo "Missing assignment"
    echo "    LCTBIN environment variable not set"
    exit 1
fi

if [[ -z ${DB} ]]
then
    echo "Missing assignment"
    echo "    DB environment variable not set"
    exit 1
fi

if [[ -z ${OBJDUMP} ]]
then
    echo "Missing assignment"
    echo "    OBJDUMP environment variable not set"
    exit 1
fi

SOURCELIST=${DB}/sourcelist.list

if [ -f ${SOURCELIST} ]
then
   rm ${SOURCELIST} 
fi

if [ -f ${SOURCELIST}.original ]
then
   rm ${SOURCELIST}.original 
fi

if [ -f ${SOURCELIST}.tmp ]
then
   rm ${SOURCELIST}.tmp
fi

EXENAME=`head -5 ${OBJDUMP} | grep "file format" | awk '{split($0,a,":"); print a[1]}'`
export DIRNAME=`dirname $EXENAME`
awk -f ${LCTBIN}/filelist.awk ${OBJDUMP} | sed 's/^[[:blank:]]*//' | sort -u > ${SOURCELIST}

cp ${SOURCELIST} ${SOURCELIST}.original

for i in `cat ${SOURCELIST}`
do
# Looking for a .i file ...
   bname=${i%.*}
   if [[ ! -z "${bname}" ]]
   then
      if [ -f "${bname}.i" ]
      then
         echo "${bname}.i" >> ${SOURCELIST}.tmp
      else
# Now look for a .ii file ...
         if [ -f "${bname}.ii" ]
         then
            echo "${bname}.ii" >> ${SOURCELIST}.tmp
#        else
#           echo "${i}" >> ${SOURCELIST}.tmp
         fi
      fi
   fi
done

TMP=0

if [ -f ${SOURCELIST}.tmp ]
then
   sort -u ${SOURCELIST}.tmp > ${SOURCELIST}
   TMP=1
fi

typeset -i -x INC
typeset -i -x BCOUNT 
INC=1
for i in `cat ${SOURCELIST}`
do
   ext=${i##*.}
   fn=${i%.*}
   if [ -f ${fn}.i ]
   then
       ext=i
   fi
   fn=${fn}.${ext}
   if [ ! -f "${fn}" ]
   then
       fn=/dev/null
   fi
   val=`grep -i "__builtin" $fn | grep -v -f ${LCTBIN}/builtin_exception.txt | wc -l` 
   let BCOUNT="${val}+0"
   if (( ${BCOUNT} > 0 ))
   then
      while [ $INC -le $BCOUNT ]
      do
         if [[ $TMP == 1 ]]
         then
            FLINE=`grep -i -n -H "__builtin" $fn 2>/dev/null | grep -v -f ${LCTBIN}/builtin_exception.txt | head -$INC | tail -1`
            TMPLINE=`echo $FLINE | awk -F ":" '{print $2}'`
            SOURCELINE=`grep "__builtin" $fn 2>/dev/null | grep -v -f ${LCTBIN}/builtin_exception.txt | head -$INC | tail -1`
            BUILTIN=`echo $SOURCELINE | awk -f ${LCTBIN}/stripbuiltin.awk`
            SOURCEFILE=`head -$TMPLINE $fn | nl | sort -rn | grep "# [0-9]* "[[:print:]]*"" | head -1 | awk '{gsub("\"","",$4); print $4}'`
	    if [ ! -f $SOURCEFILE ]
	    then
		tmpSFILE=${SOURCEFILE%.*}.i
		tmpDIR=`dirname $fn`
		if [ -f $tmpSFILE ]
		then
		    SOURCEFILE=$tmpSFILE
		else
		    if [ -f $tmpDIR/$tmpSFILE ]
		    then
			SOURCEFILE=$tmpDIR/$tmpSFILE
		    else
			if [ -f $tmpDIR/$SOURCEFILE ]
			then
			    SOURCEFILE=$tmpDIR/$SOURCEFILE
			else
			    SOURCEFILE=/dev/null
			fi
		    fi
		fi
	    fi
            GREPRES=`grep -n "$SOURCELINE" $SOURCEFILE | head -$INC | tail -1`
            SOURCELINEOUT=`echo "$GREPRES" | awk -F ":" '{print $1}'`
            if [[ ! -z "${SOURCELINEOUT}" ]]
            then
              echo -e "${SOURCEFILE}\t${SOURCELINEOUT}\tW60001\tPotentially hazardous use of builtin: $BUILTIN" >> ${DB}/tSourceMsgs.tbl
            fi
         else
            FLINE=`grep -i -n -H "__builtin" $fn 2>/dev/null | head -$INC | tail -1`
            SOURCELINE=`grep "__builtin" $fn 2>/dev/null | grep -v -f ${LCTBIN}/builtin_exception.txt | head -$INC | tail -1`
            export BUILTIN=`echo $SOURCELINE | awk -f ${LCTBIN}/stripbuiltin.awk` 
            grep -i -n -H "__builtin" $fn 2>/dev/null | head -$INC | tail -1 | awk -F ":" '{print $1"\t"$2"\tW60001\tPotentially hazardous use of builtin: "ENVIRON["BUILTIN"]}'  >> ${DB}/tSourceMsgs.tbl
         fi
         INC=`expr $INC + 1`
      done
   fi
done
